@@ -54,6 +54,12 @@ from novaclient import service_catalog
 from novaclient import utils
 
 
+# TODO(jichenjc): when an extension in contrib is moved to core extension,
+# Add the name into the following list, then after last patch merged,
+# remove the whole function
+extensions_ignored_name = ["__init__"]
+
+
 class _ClientConnectionPool(object):
 
     def __init__(self):
@@ -67,6 +73,18 @@ class _ClientConnectionPool(object):
         return self._adapters[url]
 
 
+def _log_request_id(logger, resp, service_name):
+    request_id = (resp.headers.get('x-openstack-request-id') or
+                  resp.headers.get('x-compute-request-id'))
+    if request_id:
+        logger.debug('%(method)s call to %(service_name)s for %(url)s '
+                     'used request id %(response_request_id)s',
+                     {'method': resp.request.method,
+                      'service_name': service_name,
+                      'url': resp.url,
+                      'response_request_id': request_id})
+
+
 class SessionClient(adapter.LegacyJsonAdapter):
 
     def __init__(self, *args, **kwargs):
@@ -87,6 +105,11 @@ class SessionClient(adapter.LegacyJsonAdapter):
                                                             method,
                                                             raise_exc=False,
                                                             **kwargs)
+
+        # if service name is None then use service_type for logging
+        service = self.service_name or self.service_type
+        _log_request_id(self.logger, resp, service)
+
         # TODO(andreykurilin): uncomment this line, when we will be able to
         #   check only nova-related calls
         # api_versions.check_headers(resp, self.api_version)
@@ -136,7 +159,8 @@ class HTTPClient(object):
                  http_log_debug=False, auth_system='keystone',
                  auth_plugin=None, auth_token=None,
                  cacert=None, tenant_id=None, user_id=None,
-                 connection_pool=False, api_version=None):
+                 connection_pool=False, api_version=None,
+                 logger=None):
         self.user = user
         self.user_id = user_id
         self.password = password
@@ -197,9 +221,10 @@ class HTTPClient(object):
         self.auth_plugin = auth_plugin
         self._session = None
         self._current_url = None
-        self._logger = logging.getLogger(__name__)
+        self._logger = logger or logging.getLogger(__name__)
 
-        if self.http_log_debug and not self._logger.handlers:
+        if (self.http_log_debug and logger is None and
+                not self._logger.handlers):
             # Logging level is already set on the root logger
             ch = logging.StreamHandler()
             self._logger.addHandler(ch)
@@ -314,6 +339,10 @@ class HTTPClient(object):
                                           'headers': resp.headers,
                                           'text': json.dumps(body)})
 
+        # if service name is None then use service_type for logging
+        service = self.service_name or self.service_type
+        _log_request_id(self._logger, resp, service)
+
     def open_session(self):
         if not self._connection_pool:
             self._session = requests.Session()
@@ -348,8 +377,7 @@ class HTTPClient(object):
         kwargs['headers']['Accept'] = 'application/json'
         if 'body' in kwargs:
             kwargs['headers']['Content-Type'] = 'application/json'
-            kwargs['data'] = json.dumps(kwargs['body'])
-            del kwargs['body']
+            kwargs['data'] = json.dumps(kwargs.pop('body'))
         api_versions.update_headers(kwargs["headers"], self.api_version)
         if self.timeout is not None:
             kwargs.setdefault('timeout', self.timeout)
@@ -700,6 +728,7 @@ def _construct_http_client(username=None, password=None, project_id=None,
     else:
         # FIXME(jamielennox): username and password are now optional. Need
         # to test that they were provided in this mode.
+        logger = kwargs.get('logger')
         return HTTPClient(username,
                           password,
                           user_id=user_id,
@@ -724,7 +753,8 @@ def _construct_http_client(username=None, password=None, project_id=None,
                           http_log_debug=http_log_debug,
                           cacert=cacert,
                           connection_pool=connection_pool,
-                          api_version=api_version)
+                          api_version=api_version,
+                          logger=logger)
 
 
 def discover_extensions(version, only_contrib=False):
@@ -768,7 +798,7 @@ def _discover_via_contrib_path(version):
     for ext_path in glob.iglob(ext_glob):
         name = os.path.basename(ext_path)[:-3]
 
-        if name == "__init__":
+        if name in extensions_ignored_name:
             continue
 
         module = imp.load_source(name, ext_path)
@@ -803,7 +833,8 @@ def get_client_class(version):
     return client_class
 
 
-def Client(version, *args, **kwargs):
+def Client(version, username=None, api_key=None, project_id=None,
+           auth_url=None, *args, **kwargs):
     """Initialize client object based on given version.
 
     HOW-TO:
@@ -824,7 +855,15 @@ def Client(version, *args, **kwargs):
     session API. See "The novaclient Python API" page at
     python-novaclient's doc.
     """
+    if args:
+        warnings.warn("Only VERSION, USERNAME, PASSWORD, PROJECT_ID and "
+                      "AUTH_URL arguments can be specified as positional "
+                      "arguments. All other variables should be keyword "
+                      "arguments. Note that this will become an error in "
+                      "Ocata.")
     api_version, client_class = _get_client_class_and_version(version)
     kwargs.pop("direct_use", None)
-    return client_class(api_version=api_version, direct_use=False,
+    return client_class(username=username, api_key=api_key,
+                        project_id=project_id, auth_url=auth_url,
+                        api_version=api_version, direct_use=False,
                         *args, **kwargs)
