@@ -26,7 +26,6 @@ import novaclient
 from novaclient import api_versions
 from novaclient import client as base_client
 from novaclient import exceptions
-from novaclient.i18n import _
 from novaclient.tests.unit import fakes
 from novaclient.tests.unit import utils
 from novaclient.v2 import client
@@ -46,6 +45,8 @@ CALLBACK_RE = re.compile(r"^get_http:__nova_api:8774_v\d(_\d)?$")
 # fake image uuids
 FAKE_IMAGE_UUID_1 = 'c99d7632-bd66-4be9-aed5-3dd14b223a76'
 FAKE_IMAGE_UUID_2 = 'f27f479a-ddda-419a-9bbc-d6b56b210161'
+FAKE_IMAGE_UUID_SNAPSHOT = '555cae93-fb41-4145-9c52-f5b923538a26'
+FAKE_IMAGE_UUID_SNAP_DEL = '55bb23af-97a4-4068-bdf8-f10c62880ddf'
 
 # fake request id
 FAKE_REQUEST_ID = fakes.FAKE_REQUEST_ID
@@ -55,13 +56,12 @@ FAKE_RESPONSE_HEADERS = {'x-openstack-request-id': FAKE_REQUEST_ID}
 
 class FakeClient(fakes.FakeClient, client.Client):
 
-    def __init__(self, api_version=None, *args, **kwargs):
+    def __init__(self, api_version, *args, **kwargs):
         client.Client.__init__(self, 'username', 'password',
                                'project_id', 'auth_url',
                                extensions=kwargs.get('extensions'),
                                direct_use=False)
-        self.api_version = api_version or api_versions.APIVersion("2.1")
-        kwargs["api_version"] = self.api_version
+        kwargs["api_version"] = api_version
         self.client = FakeHTTPClient(**kwargs)
 
 
@@ -129,6 +129,12 @@ class FakeHTTPClient(base_client.HTTPClient):
             # more like we'd expect when making REST calls.
             raise exceptions.NotFound('404')
 
+        # Handle fake glance v2 requests
+        v2_image = False
+        if callback.startswith('get_v2_images'):
+            v2_image = True
+            callback = callback.replace('get_v2_', 'get_')
+
         if not hasattr(self, callback):
             raise AssertionError('Called unknown API method: %s %s, '
                                  'expected fakes method name: %s' %
@@ -138,6 +144,12 @@ class FakeHTTPClient(base_client.HTTPClient):
         self.callstack.append((method, url, kwargs.get('body')))
 
         status, headers, body = getattr(self, callback)(**kwargs)
+
+        # If we're dealing with a glance v2 image response, the response
+        # isn't wrapped like the compute images API proxy is, so handle that.
+        if body and v2_image and 'image' in body:
+            body = body['image']
+
         r = utils.TestResponse({
             "status_code": status,
             "text": body,
@@ -145,7 +157,7 @@ class FakeHTTPClient(base_client.HTTPClient):
         })
         return r, body
 
-    def get_endpoint(self):
+    def get_endpoint(self, **kwargs):
         # check if endpoint matches expected format (eg, v2.1)
         if (hasattr(self, 'endpoint_type') and
                 ENDPOINT_TYPE_RE.search(self.endpoint_type)):
@@ -418,7 +430,7 @@ class FakeHTTPClient(base_client.HTTPClient):
                 "id": 5678,
                 "name": "sample-server2",
                 "image": {
-                    "id": 2,
+                    "id": FAKE_IMAGE_UUID_1,
                     "name": "sample image",
                 },
                 "flavor": {
@@ -625,29 +637,6 @@ class FakeHTTPClient(base_client.HTTPClient):
                 'rules': []}]
         })
 
-    #
-    # Server Addresses
-    #
-
-    def get_servers_1234_ips(self, **kw):
-        return (200, {}, {
-            'addresses':
-            self.get_servers_1234()[1]['server']['addresses']})
-
-    def get_servers_1234_ips_public(self, **kw):
-        return (200, {}, {
-            'public':
-            self.get_servers_1234_ips()[1]['addresses']['public']})
-
-    def get_servers_1234_ips_private(self, **kw):
-        return (
-            200, {},
-            {'private':
-                self.get_servers_1234_ips()[1]['addresses']['private']})
-
-    def delete_servers_1234_ips_public_1_2_3_4(self, **kw):
-        return (202, {}, None)
-
     #
     # Server password
     #
@@ -739,13 +728,14 @@ class FakeHTTPClient(base_client.HTTPClient):
             # raise AssertionError if we didn't find 'action' at all.
             pass
         elif action == 'os-migrateLive':
+            expected = set(['host', 'block_migration'])
+            if self.api_version >= api_versions.APIVersion("2.30"):
+                if 'force' in body[action].keys():
+                    # force can be optional
+                    expected.add('force')
             if self.api_version < api_versions.APIVersion("2.25"):
-                assert set(body[action].keys()) == set(['host',
-                                                        'block_migration',
-                                                        'disk_over_commit'])
-            else:
-                assert set(body[action].keys()) == set(['host',
-                                                        'block_migration'])
+                expected.add('disk_over_commit')
+            assert set(body[action].keys()) == expected
         elif action == 'rebuild':
             body = body[action]
             adminPass = body.get('adminPass', 'randompassword')
@@ -765,9 +755,11 @@ class FakeHTTPClient(base_client.HTTPClient):
             _body = {'adminPass': 'RescuePassword'}
         elif action == 'createImage':
             assert set(body[action].keys()) == set(['name', 'metadata'])
-            _headers = dict(location="http://blah/images/456")
+            _headers = dict(location="http://blah/images/%s" %
+                            FAKE_IMAGE_UUID_SNAPSHOT)
             if body[action]['name'] == 'mysnapshot_deleted':
-                _headers = dict(location="http://blah/images/457")
+                _headers = dict(location="http://blah/images/%s" %
+                                FAKE_IMAGE_UUID_SNAP_DEL)
         elif action == 'os-getConsoleOutput':
             assert list(body[action]) == ['length']
             return (202, {}, {'output': 'foo'})
@@ -777,7 +769,11 @@ class FakeHTTPClient(base_client.HTTPClient):
                 keys.remove('adminPass')
             if 'host' in keys:
                 keys.remove('host')
-            assert set(keys) == set(['onSharedStorage'])
+            if 'onSharedStorage' in keys:
+                keys.remove('onSharedStorage')
+            if 'force' in keys:
+                keys.remove('force')
+            assert set(keys) == set()
         else:
             raise AssertionError("Unexpected server action: %s" % action)
         _headers.update(FAKE_RESPONSE_HEADERS)
@@ -979,9 +975,6 @@ class FakeHTTPClient(base_client.HTTPClient):
     # Flavor access
     #
 
-    def get_flavors_1_os_flavor_access(self, **kw):
-        return (404, {}, None)
-
     def get_flavors_2_os_flavor_access(self, **kw):
         return (
             200, FAKE_RESPONSE_HEADERS,
@@ -996,13 +989,6 @@ class FakeHTTPClient(base_client.HTTPClient):
     # Floating IPs
     #
 
-    def get_os_floating_ip_pools(self):
-        return (
-            200,
-            {},
-            {'floating_ip_pools': [{'name': 'foo'}, {'name': 'bar'}]}
-        )
-
     def get_os_floating_ips(self, **kw):
         return (
             200,
@@ -1090,12 +1076,6 @@ class FakeHTTPClient(base_client.HTTPClient):
             {'id': 2, 'fixed_ip': '10.0.0.2', 'ip': '11.0.0.2'},
         ]})
 
-    def get_os_floating_ips_bulk_testHost(self, **kw):
-        return (200, {}, {'floating_ip_info': [
-            {'id': 1, 'fixed_ip': '10.0.0.1', 'ip': '11.0.0.1'},
-            {'id': 2, 'fixed_ip': '10.0.0.2', 'ip': '11.0.0.2'},
-        ]})
-
     def post_os_floating_ips_bulk(self, **kw):
         params = kw.get('body').get('floating_ips_bulk_create')
         pool = params.get('pool', 'defaultPool')
@@ -1114,8 +1094,6 @@ class FakeHTTPClient(base_client.HTTPClient):
     #
     def get_images(self, **kw):
         return (200, {}, {'images': [
-            {'id': 1, 'name': 'CentOS 5.2'},
-            {'id': 2, 'name': 'My Server Backup'},
             {'id': FAKE_IMAGE_UUID_1, 'name': 'CentOS 5.2'},
             {'id': FAKE_IMAGE_UUID_2, 'name': 'My Server Backup'}
         ]})
@@ -1123,18 +1101,7 @@ class FakeHTTPClient(base_client.HTTPClient):
     def get_images_detail(self, **kw):
         return (200, {}, {'images': [
             {
-                'id': 1,
-                'name': 'CentOS 5.2',
-                "updated": "2010-10-10T12:00:00Z",
-                "created": "2010-08-10T12:00:00Z",
-                "status": "ACTIVE",
-                "metadata": {
-                    "test_key": "test_value",
-                },
-                "links": {},
-            },
-            {
-                "id": 2,
+                "id": FAKE_IMAGE_UUID_SNAPSHOT,
                 "name": "My Server Backup",
                 "serverId": 1234,
                 "updated": "2010-10-10T12:00:00Z",
@@ -1144,7 +1111,7 @@ class FakeHTTPClient(base_client.HTTPClient):
                 "links": {},
             },
             {
-                "id": 3,
+                "id": FAKE_IMAGE_UUID_SNAP_DEL,
                 "name": "My Server Backup Deleted",
                 "serverId": 1234,
                 "updated": "2010-10-10T12:00:00Z",
@@ -1176,46 +1143,31 @@ class FakeHTTPClient(base_client.HTTPClient):
             }
         ]})
 
-    def get_images_1(self, **kw):
+    def get_images_555cae93_fb41_4145_9c52_f5b923538a26(self, **kw):
         return (200, {}, {'image': self.get_images_detail()[2]['images'][0]})
 
-    def get_images_2(self, **kw):
+    def get_images_55bb23af_97a4_4068_bdf8_f10c62880ddf(self, **kw):
         return (200, {}, {'image': self.get_images_detail()[2]['images'][1]})
 
-    def get_images_456(self, **kw):
-        return (200, {}, {'image': self.get_images_detail()[2]['images'][1]})
-
-    def get_images_457(self, **kw):
-        return (200, {}, {'image': self.get_images_detail()[2]['images'][2]})
-
     def get_images_c99d7632_bd66_4be9_aed5_3dd14b223a76(self, **kw):
-        return (200, {}, {'image': self.get_images_detail()[2]['images'][3]})
+        return (200, {}, {'image': self.get_images_detail()[2]['images'][2]})
 
     def get_images_f27f479a_ddda_419a_9bbc_d6b56b210161(self, **kw):
-        return (200, {}, {'image': self.get_images_detail()[2]['images'][4]})
+        return (200, {}, {'image': self.get_images_detail()[2]['images'][3]})
 
     def get_images_3e861307_73a6_4d1f_8d68_f68b03223032(self):
         raise exceptions.NotFound('404')
 
-    def post_images(self, body, **kw):
-        assert list(body) == ['image']
-        fakes.assert_has_keys(body['image'], required=['serverId', 'name'])
-        return (202, {}, self.get_images_1()[2])
-
-    def post_images_1_metadata(self, body, **kw):
+    def post_images_c99d7632_bd66_4be9_aed5_3dd14b223a76_metadata(
+            self, body, **kw):
         assert list(body) == ['metadata']
         fakes.assert_has_keys(body['metadata'],
                               required=['test_key'])
+        get_image = self.get_images_c99d7632_bd66_4be9_aed5_3dd14b223a76
         return (
             200,
             {},
-            {'metadata': self.get_images_1()[2]['image']['metadata']})
-
-    def delete_images_1(self, **kw):
-        return (204, {}, None)
-
-    def delete_images_2(self, **kw):
-        return (204, {}, None)
+            {'metadata': get_image()[2]['image']['metadata']})
 
     def delete_images_c99d7632_bd66_4be9_aed5_3dd14b223a76(self, **kw):
         return (204, {}, None)
@@ -1223,7 +1175,8 @@ class FakeHTTPClient(base_client.HTTPClient):
     def delete_images_f27f479a_ddda_419a_9bbc_d6b56b210161(self, **kw):
         return (204, {}, None)
 
-    def delete_images_1_metadata_test_key(self, **kw):
+    def delete_images_c99d7632_bd66_4be9_aed5_3dd14b223a76_metadata_test_key(
+            self, **kw):
         return (204, {}, None)
 
     #
@@ -1233,7 +1186,7 @@ class FakeHTTPClient(base_client.HTTPClient):
         return (200, {}, {'keypair':
                           self.get_os_keypairs()[2]['keypairs'][0]['keypair']})
 
-    def get_os_keypairs(self, *kw):
+    def get_os_keypairs(self, user_id=None, limit=None, marker=None, *kw):
         return (200, {}, {
             "keypairs": [{"keypair": {
                 "public_key": "FAKE_SSH_RSA",
@@ -1270,22 +1223,6 @@ class FakeHTTPClient(base_client.HTTPClient):
     # Quotas
     #
 
-    def get_os_quota_sets_test(self, **kw):
-        return (200, {}, {
-            'quota_set': {
-                'tenant_id': 'test',
-                'metadata_items': [],
-                'injected_file_content_bytes': 1,
-                'injected_file_path_bytes': 1,
-                'ram': 1,
-                'floating_ips': 1,
-                'instances': 1,
-                'injected_files': 1,
-                'cores': 1,
-                'keypairs': 1,
-                'security_groups': 1,
-                'security_group_rules': 1}})
-
     def get_os_quota_sets_tenant_id(self, **kw):
         return (200, {}, {
             'quota_set': {
@@ -1318,37 +1255,81 @@ class FakeHTTPClient(base_client.HTTPClient):
                 'security_groups': 1,
                 'security_group_rules': 1}})
 
-    def put_os_quota_sets_97f4c221_bff4_4578_b030_0df4ef119353(self, **kw):
+    def get_os_quota_sets_97f4c221bff44578b0300df4ef119353_detail(self, **kw):
         return (200, {}, {
             'quota_set': {
-                'tenant_id': '97f4c221-bff4-4578-b030-0df4ef119353',
-                'metadata_items': [],
-                'injected_file_content_bytes': 1,
-                'injected_file_path_bytes': 1,
-                'ram': 1,
-                'floating_ips': 1,
-                'instances': 1,
-                'injected_files': 1,
-                'cores': 1,
-                'keypairs': 1,
-                'security_groups': 1,
-                'security_group_rules': 1}})
-
-    def get_os_quota_sets_97f4c221_bff4_4578_b030_0df4ef119353(self, **kw):
-        return (200, {}, {
-            'quota_set': {
-                'tenant_id': '97f4c221-bff4-4578-b030-0df4ef119353',
-                'metadata_items': [],
-                'injected_file_content_bytes': 1,
-                'injected_file_path_bytes': 1,
-                'ram': 1,
-                'floating_ips': 1,
-                'instances': 1,
-                'injected_files': 1,
-                'cores': 1,
-                'keypairs': 1,
-                'security_groups': 1,
-                'security_group_rules': 1}})
+                'tenant_id': '97f4c221bff44578b0300df4ef119353',
+                'cores': {
+                    'in_use': 0,
+                    'limit': 20,
+                    'reserved': 0
+                },
+                'fixed_ips': {
+                    'in_use': 0,
+                    'limit': -1,
+                    'reserved': 0
+                },
+                'floating_ips': {
+                    'in_use': 0,
+                    'limit': 10,
+                    'reserved': 0
+                },
+                'injected_file_content_bytes': {
+                    'in_use': 0,
+                    'limit': 10240,
+                    'reserved': 0
+                },
+                'injected_file_path_bytes': {
+                    'in_use': 0,
+                    'limit': 255,
+                    'reserved': 0
+                },
+                'injected_files': {
+                    'in_use': 0,
+                    'limit': 5,
+                    'reserved': 0
+                },
+                'instances': {
+                    'in_use': 0,
+                    'limit': 10,
+                    'reserved': 0
+                },
+                'key_pairs': {
+                    'in_use': 0,
+                    'limit': 100,
+                    'reserved': 0
+                },
+                'metadata_items': {
+                    'in_use': 0,
+                    'limit': 128,
+                    'reserved': 0
+                },
+                'ram': {
+                    'in_use': 0,
+                    'limit': 51200,
+                    'reserved': 0
+                },
+                'security_group_rules': {
+                    'in_use': 0,
+                    'limit': 20,
+                    'reserved': 0
+                },
+                'security_groups': {
+                    'in_use': 0,
+                    'limit': 10,
+                    'reserved': 0
+                },
+                'server_group_members': {
+                    'in_use': 0,
+                    'limit': 10,
+                    'reserved': 0
+                },
+                'server_groups': {
+                    'in_use': 0,
+                    'limit': 10,
+                    'reserved': 0
+                }
+            }})
 
     def get_os_quota_sets_97f4c221bff44578b0300df4ef119353_defaults(self):
         return (200, {}, {
@@ -1400,9 +1381,6 @@ class FakeHTTPClient(base_client.HTTPClient):
                 'security_groups': 1,
                 'security_group_rules': 1}})
 
-    def delete_os_quota_sets_test(self, **kw):
-        return (202, {}, {})
-
     def delete_os_quota_sets_97f4c221bff44578b0300df4ef119353(self, **kw):
         return (202, {}, {})
 
@@ -1511,11 +1489,6 @@ class FakeHTTPClient(base_client.HTTPClient):
              "rules": []}
         ]})
 
-    def get_os_security_groups_1(self, **kw):
-        return (200, {}, {"security_group":
-                {'id': 1, 'name': 'test', 'description': 'FAKE_SECURITY_GROUP'}
-        })
-
     def delete_os_security_groups_1(self, **kw):
         return (202, {}, None)
 
@@ -1543,9 +1516,6 @@ class FakeHTTPClient(base_client.HTTPClient):
              'cidr': '10.0.0.0/8'}
         ]})
 
-    def delete_os_security_group_rules_1(self, **kw):
-        return (202, {}, None)
-
     def delete_os_security_group_rules_11(self, **kw):
         return (202, {}, None)
 
@@ -1575,25 +1545,6 @@ class FakeHTTPClient(base_client.HTTPClient):
              'to_port': 22, 'cidr': '10.0.0.0/8'}
         ]})
 
-    def delete_os_security_group_default_rules_1(self, **kw):
-        return (202, {}, None)
-
-    def delete_os_security_group_default_rules_11(self, **kw):
-        return (202, {}, None)
-
-    def delete_os_security_group_default_rules_12(self, **kw):
-        return (202, {}, None)
-
-    def post_os_security_group_default_rules(self, body, **kw):
-        assert list(body) == ['security_group_default_rule']
-        fakes.assert_has_keys(body['security_group_default_rule'],
-                              optional=['ip_protocol', 'from_port',
-                                        'to_port', 'cidr'])
-        rules = self.get_os_security_group_default_rules()
-        r = {'security_group_default_rule':
-             rules[2]['security_group_default_rules'][0]}
-        return (202, {}, r)
-
     #
     # Tenant Usage
     #
@@ -1686,23 +1637,6 @@ class FakeHTTPClient(base_client.HTTPClient):
                 six.u('started_at'): six.u('2012-01-20 18:06:06.479998')}],
             six.u('start'): six.u('2011-12-25 19:48:41.750687'),
             six.u('total_local_gb_usage'): 0.0}})
-    #
-    # Certificates
-    #
-
-    def get_os_certificates_root(self, **kw):
-        return (
-            200,
-            {},
-            {'certificate': {'private_key': None, 'data': 'foo'}}
-        )
-
-    def post_os_certificates(self, **kw):
-        return (
-            200,
-            {},
-            {'certificate': {'private_key': 'foo', 'data': 'bar'}}
-        )
 
     #
     # Aggregates
@@ -1741,18 +1675,9 @@ class FakeHTTPClient(base_client.HTTPClient):
     def put_os_aggregates_1(self, body, **kw):
         return self._return_aggregate()
 
-    def put_os_aggregates_2(self, body, **kw):
-        return self._return_aggregate()
-
-    def put_os_aggregates_3(self, body, **kw):
-        return self._return_aggregate_3()
-
     def post_os_aggregates_1_action(self, body, **kw):
         return self._return_aggregate()
 
-    def post_os_aggregates_2_action(self, body, **kw):
-        return self._return_aggregate()
-
     def post_os_aggregates_3_action(self, body, **kw):
         return self._return_aggregate_3()
 
@@ -1825,16 +1750,6 @@ class FakeHTTPClient(base_client.HTTPClient):
     #
     # Hosts
     #
-    def get_os_hosts_host(self, *kw):
-        return (200, {}, {'host':
-                [{'resource': {'project': '(total)', 'host': 'dummy',
-                  'cpu': 16, 'memory_mb': 32234, 'disk_gb': 128}},
-                 {'resource': {'project': '(used_now)', 'host': 'dummy',
-                  'cpu': 1, 'memory_mb': 2075, 'disk_gb': 45}},
-                 {'resource': {'project': '(used_max)', 'host': 'dummy',
-                  'cpu': 1, 'memory_mb': 2048, 'disk_gb': 30}},
-                 {'resource': {'project': 'admin', 'host': 'dummy',
-                  'cpu': 1, 'memory_mb': 2048, 'disk_gb': 30}}]})
 
     def get_os_hosts(self, **kw):
         zone = kw.get('zone', 'nova1')
@@ -1845,9 +1760,6 @@ class FakeHTTPClient(base_client.HTTPClient):
                                      'service': 'nova-cert',
                                      'zone': zone}]})
 
-    def get_os_hosts_sample_host(self, *kw):
-        return (200, {}, {'host': [{'resource': {'host': 'sample_host'}}], })
-
     def put_os_hosts_sample_host_1(self, body, **kw):
         return (200, {}, {'host': 'sample-host_1',
                           'status': 'enabled'})
@@ -1873,56 +1785,12 @@ class FakeHTTPClient(base_client.HTTPClient):
         return (200, {}, {'host': 'sample_host',
                           'power_action': 'shutdown'})
 
-    def put_os_hosts_sample_host(self, body, **kw):
-        result = {'host': 'dummy'}
-        result.update(body)
-        return (200, {}, result)
-
     def get_os_hypervisors(self, **kw):
         return (200, {}, {
             "hypervisors": [
                 {'id': 1234, 'hypervisor_hostname': 'hyper1'},
                 {'id': 5678, 'hypervisor_hostname': 'hyper2'}]})
 
-    def get_os_hypervisors_detail(self, **kw):
-        return (200, {}, {
-            "hypervisors": [
-                {'id': 1234,
-                 'service': {'id': 1, 'host': 'compute1'},
-                 'vcpus': 4,
-                 'memory_mb': 10 * 1024,
-                 'local_gb': 250,
-                 'vcpus_used': 2,
-                 'memory_mb_used': 5 * 1024,
-                 'local_gb_used': 125,
-                 'hypervisor_type': "xen",
-                 'hypervisor_version': 3,
-                 'hypervisor_hostname': "hyper1",
-                 'free_ram_mb': 5 * 1024,
-                 'free_disk_gb': 125,
-                 'current_workload': 2,
-                 'running_vms': 2,
-                 'cpu_info': 'cpu_info',
-                 'disk_available_least': 100},
-                {'id': 2,
-                 'service': {'id': 2, 'host': "compute2"},
-                 'vcpus': 4,
-                 'memory_mb': 10 * 1024,
-                 'local_gb': 250,
-                 'vcpus_used': 2,
-                 'memory_mb_used': 5 * 1024,
-                 'local_gb_used': 125,
-                 'hypervisor_type': "xen",
-                 'hypervisor_version': 3,
-                 'hypervisor_hostname': "hyper2",
-                 'free_ram_mb': 5 * 1024,
-                 'free_disk_gb': 125,
-                 'current_workload': 2,
-                 'running_vms': 2,
-                 'cpu_info': 'cpu_info',
-                 'disk_available_least': 100}]
-        })
-
     def get_os_hypervisors_statistics(self, **kw):
         return (200, {}, {
             "hypervisor_statistics": {
@@ -2040,6 +1908,50 @@ class FakeHTTPClient(base_client.HTTPClient):
                            'hypervisor_hostname': "hyper1",
                            'uptime': "fake uptime"}})
 
+    def get_v2_0_networks(self, **kw):
+        """Return neutron proxied networks.
+
+        We establish a few different possible networks that we can get
+        by name, which we can then call in tests. The only usage of
+        this API should be for name -> id translation, however a full
+        valid neutron block is provided for the private network to see
+        the kinds of things that will be in that payload.
+        """
+
+        name = kw.get('name', "blank")
+
+        networks_by_name = {
+            'private': [
+                {"status": "ACTIVE",
+                 "router:external": False,
+                 "availability_zone_hints": [],
+                 "availability_zones": ["nova"],
+                 "description": "",
+                 "name": "private",
+                 "subnets": ["64706c26-336c-4048-ab3c-23e3283bca2c",
+                             "18512740-c760-4d5f-921f-668105c9ee44"],
+                 "shared": False,
+                 "tenant_id": "abd42f270bca43ea80fe4a166bc3536c",
+                 "created_at": "2016-08-15T17:34:49",
+                 "tags": [],
+                 "ipv6_address_scope": None,
+                 "updated_at": "2016-08-15T17:34:49",
+                 "admin_state_up": True,
+                 "ipv4_address_scope": None,
+                 "port_security_enabled": True,
+                 "mtu": 1450,
+                 "id": "e43a56c7-11d4-45c9-8681-ddc8171b5850",
+                 "revision": 2}],
+            'duplicate': [
+                {"status": "ACTIVE",
+                 "id": "e43a56c7-11d4-45c9-8681-ddc8171b5850"},
+                {"status": "ACTIVE",
+                 "id": "f43a56c7-11d4-45c9-8681-ddc8171b5850"}],
+            'blank': []
+        }
+
+        return (200, {}, {"networks": networks_by_name[name]})
+
     def get_os_networks(self, **kw):
         return (200, {}, {'networks': [{"label": "1", "cidr": "10.0.0.0/24",
                                         'project_id':
@@ -2052,54 +1964,12 @@ class FakeHTTPClient(base_client.HTTPClient):
     def post_os_networks(self, **kw):
         return (202, {}, {'network': kw})
 
-    def get_os_networks_1(self, **kw):
-        return (200, {}, {'network': {"label": "1", "cidr": "10.0.0.0/24",
-                                      "id": "1"}})
-
-    def delete_os_networks_networkdelete(self, **kw):
-        return (202, {}, None)
-
     def post_os_networks_add(self, **kw):
         return (202, {}, None)
 
-    def post_os_networks_networkdisassociate_action(self, **kw):
-        return (202, {}, None)
-
-    def get_os_fping(self, **kw):
-        return (
-            200, {}, {
-                'servers': [
-                    {
-                        "id": "1",
-                        "project_id": "fake-project",
-                        "alive": True,
-                    },
-                    {
-                        "id": "2",
-                        "project_id": "fake-project",
-                        "alive": True,
-                    },
-                ]
-            }
-        )
-
-    def get_os_fping_1(self, **kw):
-        return (
-            200, {}, {
-                'server': {
-                    "id": "1",
-                    "project_id": "fake-project",
-                    "alive": True,
-                }
-            }
-        )
-
     def post_os_networks_1_action(self, **kw):
         return (202, {}, None)
 
-    def post_os_networks_networktest_action(self, **kw):
-        return (202, {}, None)
-
     def post_os_networks_2_action(self, **kw):
         return (202, {}, None)
 
@@ -2120,17 +1990,6 @@ class FakeHTTPClient(base_client.HTTPClient):
     def delete_os_tenant_networks_1(self, **kw):
         return (202, {}, None)
 
-    def get_os_availability_zone(self, **kw):
-        return (200, {}, {
-            "availabilityZoneInfo": [
-                {"zoneName": "zone-1",
-                 "zoneState": {"available": True},
-                 "hosts": None},
-                {"zoneName": "zone-2",
-                 "zoneState": {"available": False},
-                 "hosts": None}]
-        })
-
     def get_os_availability_zone_detail(self, **kw):
         return (200, {}, {
             "availabilityZoneInfo": [
@@ -2432,28 +2291,11 @@ class FakeHTTPClient(base_client.HTTPClient):
     def post_os_server_groups(self, body, **kw):
         return self._return_server_group()
 
-    def get_os_server_groups_2cbd51f4_fafe_4cdb_801b_cf913a6f288b(self, **kw):
-        return self._return_server_group()
-
-    def put_os_server_groups_2cbd51f4_fafe_4cdb_801b_cf913a6f288b(self, **kw):
-        return self._return_server_group()
-
-    def post_os_server_groups_2cbd51f4_fafe_4cdb_801b_cf913a6f288b_action(
-            self, body, **kw):
-        return self._return_server_group()
-
-    def delete_os_server_groups_2cbd51f4_fafe_4cdb_801b_cf913a6f288b(
-            self, **kw):
-        return (202, {}, None)
-
     def post_servers_1234_migrations_1_action(self, body):
         return (202, {}, None)
 
+    @api_versions.wraps(start_version="2.23")
     def get_servers_1234_migrations_1(self, **kw):
-        # TODO(Shaohe Feng) this condition check can be a decorator
-        if self.api_version < api_versions.APIVersion("2.23"):
-            raise exceptions.UnsupportedVersion(_("Unsupport version %s")
-                                                % self.api_version)
         migration = {"migration": {
             "created_at": "2016-01-29T13:42:02.000000",
             "dest_compute": "compute2",
@@ -2474,11 +2316,8 @@ class FakeHTTPClient(base_client.HTTPClient):
         }}
         return (200, FAKE_RESPONSE_HEADERS, migration)
 
+    @api_versions.wraps(start_version="2.23")
     def get_servers_1234_migrations(self, **kw):
-        # TODO(Shaohe Feng) this condition check can be a decorator
-        if self.api_version < api_versions.APIVersion("2.23"):
-            raise exceptions.UnsupportedVersion(_("Unsupport version %s")
-                                                % self.api_version)
         migrations = {'migrations': [
             {
                 "created_at": "2016-01-29T13:42:02.000000",
@@ -2503,6 +2342,24 @@ class FakeHTTPClient(base_client.HTTPClient):
     def delete_servers_1234_migrations_1(self):
         return (202, {}, None)
 
+    def put_servers_1234_tags_tag(self, **kw):
+        return (201, {}, None)
+
+    def put_servers_1234_tags(self, **kw):
+        return (201, {}, None)
+
+    def get_servers_1234_tags(self, **kw):
+        return (200, {}, {'tags': ['tag1', 'tag2']})
+
+    def delete_servers_1234_tags_tag(self, **kw):
+        return (204, {}, None)
+
+    def delete_servers_1234_tags(self, **kw):
+        return (204, {}, None)
+
+    def get_servers_1234_tags_tag(self, **kw):
+        return (204, {}, None)
+
 
 class FakeSessionClient(fakes.FakeClient, client.Client):
 
@@ -2510,8 +2367,8 @@ class FakeSessionClient(fakes.FakeClient, client.Client):
         client.Client.__init__(self, 'username', 'password',
                                'project_id', 'auth_url',
                                extensions=kwargs.get('extensions'),
-                               api_version=api_version, direct_use=False)
-        kwargs['api_version'] = api_version
+                               direct_use=False)
+        kwargs["api_version"] = api_version
         self.client = FakeSessionMockClient(**kwargs)
 
 
